body {
  font-family: "Corbel";
  font-size: 1.16em;
}

h1 {
  font-weight: bold;
  color: #660033;
}

h2 {
  font-weight: bold;
  font-size: 1.15em;
}

a {
  color: #ff784b;
}
a:hover {
  color: #ff784b;
}

#logo {
  height: 96px;
  width: 96px;
  background-image: url("logo_saem.png");
}

#tools-group {
  background-image: url("bandeau_saem.jpg");
}

#tools-group, #pagefooter, .panel, table, .breadcrumb, .nav-tabs {
  font-family: "Helvetica Neue", Helvetica, Arial, sans-serif;
  font-size: 1.08em;
}

.navbar-default {
	background-color: #ff6633;
}

.navbar-default .navbar-nav li {
	padding: 0 1px;
	margin: 0 1px;
}

.navbar-default .navbar-nav li a {
  color: #660033;
	background-color: #cccccc;
	padding: 5px 15px 10px 15px;
	border-bottom-left-radius: 8px;
	border-bottom-right-radius: 8px;
}

.navbar-default .navbar-nav .dropdown-menu li a {
  color: #660033;
	background: transparent;
	border-radius: 8px;
}

.navbar-default .navbar-nav li a:hover {
	background-color: #dddddd;
}

.similar_values-category {
    font-weight: bold;
    padding: .2em .4em;
    margin: .8em 0 .2em;
    line-height: 1.5;
 }

.similar_values-list {
    background: #F5F5F5;
}

.highlight {
    background-color: #f6a048;
}

.selected {
    font-weight: bold;
}

.seda-export-error {
    color: #C11B17;
    font-size: 80%;
}

div.formGroup {
    border: 1px solid #BBB;
    padding-left: 3px;
    padding-right: 3px;
}

.form-control.metaField, .form-control.hasDatepicker {
    width: 90%;
    display: inline;
}

.form-control.metaField.hasDatepicker{
    width: 86%;
}

.panel .list-group .list-group-item {
    padding: .2em 1em;
}

ul.navbar-nav {
    padding-left: 1em;
}

ul.breadcrumb {
    padding-left: 0px;
}

.cw-breadcrumb {
  background-color: #f5f5f5;
}

.cw-breadcrumb ul a {
  color: #ff784b;
}

.cw-breadcrumb ul li.active a {
  color: #ff784b;
}

.cw-breadcrumb ul a:hover {
  text-decoration: underline;
}

.panel-info {
  border-color: #757575;
}

.panel-info .panel-heading {
  color: #ffffff;
  background-color: #757575;
}

.panel-info .panel-heading a {
  color: inherit;
}

.panel-info .panel-heading a:hover {
  color: #ff784b;
  text-decoration: none;
}

.panel-default .panel-heading {
  font-weight: bold;
	background-color: #cccccc;
}

div.facet div.facetTitle {
  font-weight: normal;
	background-color: #cccccc;
}
